searchData={"items":[{"type":"module","title":"MapSorter","doc":"Sorts a list of maps per a list of sort specs.\n\nAlso supports:\n\n- keywords\n- structs implementing the `Access` behaviour\n- nested maps, keywords or structs implementing the `Access` behaviour","ref":"MapSorter.html"},{"type":"macro","title":"MapSorter.sort/2","doc":"Sorts `maps` per the given `sort_specs`.\n\nExamples of sort specs for flat data structures:\n```\n- implicit: [:dob, :name]\n- mixed:    [:dob, desc: :name]\n- explicit: [asc: :dob, desc: :name]\n```\n\nExamples of sort specs with a `Date` key for flat data structures:\n```\n- implicit: [{:dob, Date}, :name]\n- mixed:    [{:dob, Date}, desc: :name]\n- explicit: [asc: {:dob, Date}, desc: :name]\n```\n\nExamples of sort specs for nested data structures:\n```\n- implicit: [[:birth, :date], :name]\n- mixed   : [[:birth, :date], desc: :name]\n- explicit: [asc: [:birth, :date], desc: :name]\n```\n\nExamples of sort specs with a `Date` key for nested data structures:\n```\n- implicit: [{[:birth, :date], Date}, :name]\n- mixed:    [{[:birth, :date], Date}, desc: :name]\n- explicit: [asc: {[:birth, :date], Date}, desc: :name]\n```","ref":"MapSorter.html#sort/2"},{"type":"macro","title":"Examples - MapSorter.sort/2","doc":"iex> require MapSorter\n    iex> people = [\n    ...>   %{name: \"Mike\", likes: \"movies\" , dob: \"1992-04-15\"},\n    ...>   %{name: \"Mary\", likes: \"travels\", dob: \"1992-04-15\"},\n    ...>   %{name: \"Bill\", likes: \"karate\" , dob: \"1977-08-28\"},\n    ...>   %{name: \"Joe\" , likes: \"boxing\" , dob: \"1977-08-28\"},\n    ...>   %{name: \"Jill\", likes: \"cooking\", dob: \"1976-09-28\"}\n    ...> ]\n    iex> sort_specs = Tuple.to_list({:dob, {:desc, :likes}})\n    iex> sorted_people = %{\n    ...>   explicit: MapSorter.sort(people, asc: :dob, desc: :likes),\n    ...>   mixed:    MapSorter.sort(people, [:dob, desc: :likes]),\n    ...>   runtime:  MapSorter.sort(people, sort_specs)\n    ...> }\n    iex> sorted_people.explicit == sorted_people.mixed and\n    ...> sorted_people.explicit == sorted_people.runtime and\n    ...> sorted_people.explicit\n    [\n      %{name: \"Jill\", likes: \"cooking\", dob: \"1976-09-28\"},\n      %{name: \"Bill\", likes: \"karate\" , dob: \"1977-08-28\"},\n      %{name: \"Joe\" , likes: \"boxing\" , dob: \"1977-08-28\"},\n      %{name: \"Mary\", likes: \"travels\", dob: \"1992-04-15\"},\n      %{name: \"Mike\", likes: \"movies\" , dob: \"1992-04-15\"}\n    ]","ref":"MapSorter.html#sort/2-examples"},{"type":"module","title":"MapSorter.Compare","doc":"A compare function and a heredoc to become a compare function.","ref":"MapSorter.Compare.html"},{"type":"type","title":"MapSorter.Compare.comp_fun/0","doc":"Compare function","ref":"MapSorter.Compare.html#t:comp_fun/0"},{"type":"function","title":"MapSorter.Compare.fun/1","doc":"Generates an `Enum.sort/2` compare function from a list of sort specs.","ref":"MapSorter.Compare.html#fun/1"},{"type":"function","title":"Examples - MapSorter.Compare.fun/1","doc":"iex> alias MapSorter.Compare\n    iex> sort_specs = [:dob, desc: :likes]\n    iex> fun = Compare.fun(sort_specs)\n    iex> is_function(fun, 2)\n    true\n\n    # Compare function always true => no reordering...\n    iex> alias MapSorter.Compare\n    iex> sort_specs = {:dob, desc: :likes}\n    iex> fun = Compare.fun(sort_specs)\n    iex> is_function(fun, 2) and fun.(1, 2) and fun.(:any, ~c'any')\n    true\n\n    iex> alias MapSorter.Compare\n    iex> sort_specs = fn -> [:dob, desc: :likes] end\n    iex> fun = Compare.fun(sort_specs.())\n    iex> is_function(fun, 2)\n    true\n\n    iex> alias MapSorter.Compare\n    iex> sort_specs = Tuple.to_list({:dob, {:desc, :likes}})\n    iex> fun = Compare.fun(sort_specs)\n    iex> is_function(fun, 2)\n    true","ref":"MapSorter.Compare.html#fun/1-examples"},{"type":"function","title":"MapSorter.Compare.heredoc/1","doc":"Generates a `cond/1` expression as a heredoc to become a compare function.\n\nThe heredoc may be converted into its quoted form at compile time or else\nhave its contents evaluated at runtime.\n\nThis function cannot be named `cond` as it is among the `Kernel.SpecialForms`.","ref":"MapSorter.Compare.html#heredoc/1"},{"type":"function","title":"Examples - MapSorter.Compare.heredoc/1","doc":"iex> alias MapSorter.Compare\n    iex> Compare.heredoc([])\n    \"\"\"\n    & cond do\n    true -> true or &1 * &2\n    end\n    \"\"\"\n\n    iex> alias MapSorter.Compare\n    iex> sort_specs = [:name, {:desc, :dob}]\n    iex> Compare.heredoc(sort_specs)\n    \"\"\"\n    & cond do\n    &1[:name] < &2[:name] -> true\n    &1[:name] > &2[:name] -> false\n    &1[:dob] > &2[:dob] -> true\n    &1[:dob] < &2[:dob] -> false\n    true -> true or &1 * &2\n    end\n    \"\"\"\n\n    iex> alias MapSorter.Compare\n    iex> sort_specs = [:name, {:desc, {:dob, Date}}]\n    iex> Compare.heredoc(sort_specs)\n    \"\"\"\n    & cond do\n    &1[:name] < &2[:name] -> true\n    &1[:name] > &2[:name] -> false\n    &1[:dob] != nil and Date.compare(&1[:dob], &2[:dob]) == :gt -> true\n    &1[:dob] != nil and Date.compare(&1[:dob], &2[:dob]) == :lt -> false\n    true -> true or &1 * &2\n    end\n    \"\"\"\n\n    iex> alias MapSorter.Compare\n    iex> sort_specs = [:name, {:desc, {:account, Path}}]\n    iex> Compare.heredoc(sort_specs)\n    \"\"\"\n    & cond do\n    &1[:name] < &2[:name] -> true\n    &1[:name] > &2[:name] -> false\n    &1[{:account, Path}] > &2[{:account, Path}] -> true\n    &1[{:account, Path}] < &2[{:account, Path}] -> false\n    true -> true or &1 * &2\n    end\n    \"\"\"","ref":"MapSorter.Compare.html#heredoc/1-examples"},{"type":"module","title":"MapSorter.Cond","doc":"Generates `cond/1` clauses as a heredoc from a list of sort specs.","ref":"MapSorter.Cond.html"},{"type":"function","title":"MapSorter.Cond.clauses/1","doc":"Generates `cond/1` clauses as a heredoc from a list of sort specs.","ref":"MapSorter.Cond.html#clauses/1"},{"type":"function","title":"Examples - MapSorter.Cond.clauses/1","doc":"iex> alias MapSorter.Cond\n    iex> sort_specs = [:name, {:desc, :dob}]\n    iex> Cond.clauses(sort_specs)\n    \"\"\"\n    &1[:name] < &2[:name] -> true\n    &1[:name] > &2[:name] -> false\n    &1[:dob] > &2[:dob] -> true\n    &1[:dob] < &2[:dob] -> false\n    \"\"\"\n\n    iex> alias MapSorter.Cond\n    iex> sort_specs = [:name, {:desc, {:dob, Date}}]\n    iex> Cond.clauses(sort_specs)\n    \"\"\"\n    &1[:name] < &2[:name] -> true\n    &1[:name] > &2[:name] -> false\n    &1[:dob] != nil and Date.compare(&1[:dob], &2[:dob]) == :gt -> true\n    &1[:dob] != nil and Date.compare(&1[:dob], &2[:dob]) == :lt -> false\n    \"\"\"\n\n    iex> alias MapSorter.Cond\n    iex> sort_specs = [:name, {:desc, {:dob, String}}]\n    iex> Cond.clauses(sort_specs)\n    \"\"\"\n    &1[:name] < &2[:name] -> true\n    &1[:name] > &2[:name] -> false\n    &1[{:dob, String}] > &2[{:dob, String}] -> true\n    &1[{:dob, String}] < &2[{:dob, String}] -> false\n    \"\"\"","ref":"MapSorter.Cond.html#clauses/1-examples"},{"type":"module","title":"MapSorter.Log","doc":"","ref":"MapSorter.Log.html"},{"type":"function","title":"MapSorter.Log.debug/2","doc":"","ref":"MapSorter.Log.html#debug/2"},{"type":"function","title":"MapSorter.Log.warning/2","doc":"","ref":"MapSorter.Log.html#warning/2"},{"type":"module","title":"MapSorter.Regex","doc":"","ref":"MapSorter.Regex.html"},{"type":"function","title":"MapSorter.Regex.compare/2","doc":"Compares two regex structs.","ref":"MapSorter.Regex.html#compare/2"},{"type":"module","title":"MapSorter.SortSpec","doc":"Defines sort spec types and generates square brackets access syntax strings.","ref":"MapSorter.SortSpec.html"},{"type":"function","title":"MapSorter.SortSpec.brackets/1","doc":"Generates a square brackets access syntax string from a sort spec `key`.","ref":"MapSorter.SortSpec.html#brackets/1"},{"type":"function","title":"Examples - MapSorter.SortSpec.brackets/1","doc":"iex> import MapSorter.SortSpec, only: [brackets: 1]\n    iex> brackets([:birth, :date])\n    \"[:birth][:date]\"\n\n    iex> import MapSorter.SortSpec, only: [brackets: 1]\n    iex> brackets([:address, CITY, :state])\n    ~s<[:address][CITY][:state]>\n\n    iex> import MapSorter.SortSpec, only: [brackets: 1]\n    iex> brackets([:dob])\n    \"[:dob]\"\n\n    iex> import MapSorter.SortSpec, only: [brackets: 1]\n    iex> brackets(:dob)\n    \"[:dob]\"\n\n    iex> import MapSorter.SortSpec, only: [brackets: 1]\n    iex> brackets(DOB)\n    \"[DOB]\"\n\n    iex> import MapSorter.SortSpec, only: [brackets: 1]\n    iex> brackets([1, <<1, 2, 3>>, 3.14, DOB])\n    ~s{[1][<<1, 2, 3>>][3.14][DOB]}\n\n    iex> import MapSorter.SortSpec, only: [brackets: 1]\n    iex> brackets([DOB, \"3.14\", 3.14, 0, {1, 2}, :likes])\n    ~s<[DOB][\"3.14\"][3.14][0][{1, 2}][:likes]>","ref":"MapSorter.SortSpec.html#brackets/1-examples"},{"type":"type","title":"MapSorter.SortSpec.key/0","doc":"Sort spec key","ref":"MapSorter.SortSpec.html#t:key/0"},{"type":"type","title":"MapSorter.SortSpec.sort_dir/0","doc":"Sort direction","ref":"MapSorter.SortSpec.html#t:sort_dir/0"},{"type":"type","title":"MapSorter.SortSpec.t/0","doc":"Sort spec","ref":"MapSorter.SortSpec.html#t:t/0"},{"type":"module","title":"MapSorter.SortSpecs","doc":"Converts the AST of sort specs into the AST of an `Enum.sort/2` compare\r\nfunction.","ref":"MapSorter.SortSpecs.html"},{"type":"type","title":"MapSorter.SortSpecs.t/0","doc":"A list of sort specs","ref":"MapSorter.SortSpecs.html#t:t/0"},{"type":"function","title":"MapSorter.SortSpecs.to_quoted/1","doc":"Converts `sort_specs` into the AST of an `Enum.sort/2` compare function.","ref":"MapSorter.SortSpecs.html#to_quoted/1"},{"type":"function","title":"Examples - MapSorter.SortSpecs.to_quoted/1","doc":"# Compile time sort specs...\r\n    iex> alias MapSorter.SortSpecs\r\n    iex> sort_specs = [:dob]\r\n    iex> {:ok, comp_fun_ast} = SortSpecs.to_quoted(sort_specs)\r\n    iex> {:&, _meta, [args]} = comp_fun_ast\r\n    iex> {comp_fun, []} = Code.eval_quoted(comp_fun_ast)\r\n    iex> is_function(comp_fun, 2) and Macro.to_string(args)\r\n    \"\"\"\r\n    cond do\r\n      &1[:dob] < &2[:dob] -> true\r\n      &1[:dob] > &2[:dob] -> false\r\n      true -> true or &1 * &2\r\n    end\\\r\n    \"\"\"\r\n\r\n    # Compile time sort specs...\r\n    iex> alias MapSorter.SortSpecs\r\n    iex> sort_specs = fn -> [desc: :dob, asc: :bmi] end\r\n    iex> {:ok, comp_fun_ast} = sort_specs.() |> SortSpecs.to_quoted()\r\n    iex> {:&, _meta, [args]} = comp_fun_ast\r\n    iex> {comp_fun, []} = Code.eval_quoted(comp_fun_ast)\r\n    iex> is_function(comp_fun, 2) and Macro.to_string(args)\r\n    \"\"\"\r\n    cond do\r\n      &1[:dob] > &2[:dob] -> true\r\n      &1[:dob] < &2[:dob] -> false\r\n      &1[:bmi] < &2[:bmi] -> true\r\n      &1[:bmi] > &2[:bmi] -> false\r\n      true -> true or &1 * &2\r\n    end\\\r\n    \"\"\"\r\n\r\n    # Compile time sort specs...\r\n    iex> alias MapSorter.SortSpecs\r\n    iex> sort_specs = [:dob, desc: :likes]\r\n    iex> {:ok, comp_fun_ast} = SortSpecs.to_quoted(sort_specs)\r\n    iex> {:&, _meta, [args]} = comp_fun_ast\r\n    iex> {comp_fun, []} = Code.eval_quoted(comp_fun_ast)\r\n    iex> is_function(comp_fun, 2) and Macro.to_string(args)\r\n    \"\"\"\r\n    cond do\r\n      &1[:dob] < &2[:dob] -> true\r\n      &1[:dob] > &2[:dob] -> false\r\n      &1[:likes] > &2[:likes] -> true\r\n      &1[:likes] < &2[:likes] -> false\r\n      true -> true or &1 * &2\r\n    end\\\r\n    \"\"\"\r\n\r\n    # Runtime sort specs...\r\n    iex> alias MapSorter.SortSpecs\r\n    iex> sort_specs = quote do: Tuple.to_list({:dob, {:desc, :likes}})\r\n    iex> {:ok, comp_fun_ast} = SortSpecs.to_quoted(sort_specs)\r\n    iex> {{:., _, nested_args}, _meta, [args]} = comp_fun_ast\r\n    iex> {comp_fun, []} = Code.eval_quoted(comp_fun_ast)\r\n    iex> is_function(comp_fun, 2) and\r\n    ...> {Macro.to_string(nested_args), Macro.to_string(args)}\r\n    {\"[MapSorter.Compare, :fun]\", \"Tuple.to_list({:dob, {:desc, :likes}})\"}","ref":"MapSorter.SortSpecs.html#to_quoted/1-examples"},{"type":"module","title":"MapSorter.TestSetup","doc":"","ref":"MapSorter.TestSetup.html"},{"type":"function","title":"MapSorter.TestSetup.setup_all/1","doc":"","ref":"MapSorter.TestSetup.html#setup_all/1"},{"type":"module","title":"MapSorter.TestSetup.Person","doc":"","ref":"MapSorter.TestSetup.Person.html"},{"type":"function","title":"MapSorter.TestSetup.Person.fetch/2","doc":"Callback implementation for  Access.fetch/2 .","ref":"MapSorter.TestSetup.Person.html#fetch/2"},{"type":"function","title":"MapSorter.TestSetup.Person.get_and_update/3","doc":"Callback implementation for  Access.get_and_update/3 .","ref":"MapSorter.TestSetup.Person.html#get_and_update/3"},{"type":"function","title":"MapSorter.TestSetup.Person.pop/2","doc":"Callback implementation for  Access.pop/2 .","ref":"MapSorter.TestSetup.Person.html#pop/2"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.38.2"}}